﻿/*
TrendScalper cBot for M5

Conversation in forum
https://www.forexfactory.com/thread/1026888-trend-following-scalping-method-h1-m5

Video 1
https://youtu.be/zhEukjCzXwM

Video 2
https://youtu.be/0hUJTsjYPcQ

*/
using System;
using System.Linq;
using cAlgo.API;
using cAlgo.API.Indicators;
using cAlgo.API.Internals;
using cAlgo.Indicators;

namespace cAlgo.Robots
{
    [Robot(TimeZone = TimeZones.UTC, AccessRights = AccessRights.None)]
    public class TrendScalpercBot : Robot
    {
        // Choose if you want to use fixed lot size or risk %
        [Parameter("Method", Group = "Money management", DefaultValue = MMType.FIXED_LOT)]
        public MMType ParamMoneyMgmntType { get; set; }

        // fIXED LOT SIZE
        [Parameter("Lot size", Group = "Money management", DefaultValue = 0.01, MinValue = 0.01)]
        public double ParamLotSize { get; set; }

        // pERCENT OF THE ACCOUNT THAT YOU WANT TO RISK IN EACH OPERATION
        [Parameter("Risk %", Group = "Money management", DefaultValue = 2, MinValue = 0.1, MaxValue = 50.0)]
        public double ParamRisk { get; set; }

        // ===============================================

        [Parameter("Use restricted trading hours", Group = "Trading hours (GMT)", DefaultValue = true)]
        public bool ParamRestrictedTradingHours { get; set; }

        [Parameter("GMT start hour", Group = "Trading hours (GMT)", DefaultValue = 8, MinValue = 0, MaxValue = 23)]
        public int ParamStartHour { get; set; }

        [Parameter("GMT start minute", Group = "Trading hours (GMT)", DefaultValue = 0, MinValue = 0, MaxValue = 59)]
        public int ParamStartMinute { get; set; }

        [Parameter("GMT end hour", Group = "Trading hours (GMT)", DefaultValue = 17, MinValue = 0, MaxValue = 23)]
        public int ParamEndHour { get; set; }

        [Parameter("GMT end minute", Group = "Trading hours (GMT)", DefaultValue = 0, MinValue = 0, MaxValue = 59)]
        public int ParamEndMinute { get; set; }

        [Parameter("Adjust summer time", Group = "Trading hours (GMT)", DefaultValue = true)]
        public bool ParamAdjustSummerTime { get; set; }



        private int status;
        // Estado a la espera sin posicion
        private const int STATUS_IDLE = 0;
        // Estado con ordenes stop
        private const int STATUS_WAITING = 1;
        // Estado con posicion abierta
        private const int STATUS_TRADING = 2;

        private ExponentialMovingAverage ema21H1;
        private ExponentialMovingAverage ema8H1;
        private ExponentialMovingAverage ema8M5;
        private ExponentialMovingAverage ema13M5;
        private ExponentialMovingAverage ema21M5;

        private double volume;
        private double stopLossPrice;
        private PendingOrder pendingOrder1;
        private PendingOrder pendingOrder2;
        private Position position1;
        private Position position2;

        private string label = "TrendScalper cBot";
        private string version = "1.0";

        protected override void OnStart()
        {
            if (Bars.TimeFrame != TimeFrame.Minute5)
            {
                Print("Error. This robot must be used only in M5.");
                throw new Exception("Error. This robot must be used only in M5.");
            }
            Positions.Closed += this.OnClosedPosition;
            PendingOrders.Filled += this.OnPendingOrderFilled;

            this.status = STATUS_IDLE;
            Bars barsH1 = MarketData.GetBars(TimeFrame.Hour, Symbol.Name);

            this.ema8H1 = Indicators.ExponentialMovingAverage(barsH1.ClosePrices, 8);
            this.ema21H1 = Indicators.ExponentialMovingAverage(barsH1.ClosePrices, 21);

            this.ema8M5 = Indicators.ExponentialMovingAverage(Bars.ClosePrices, 8);
            this.ema13M5 = Indicators.ExponentialMovingAverage(Bars.ClosePrices, 13);
            this.ema21M5 = Indicators.ExponentialMovingAverage(Bars.ClosePrices, 21);

            this.volume = Symbol.QuantityToVolumeInUnits(this.ParamLotSize);

            if (this.RunningMode == RunningMode.RealTime || this.RunningMode == RunningMode.VisualBacktesting)
            {
                Chart.DrawStaticText("version", "TrendScalper cBot " + this.version, VerticalAlignment.Bottom, HorizontalAlignment.Left, Chart.ColorSettings.ForegroundColor);
            }
            Print("Started " + this.label + " version " + this.version);
        }

        protected void OnTickxx()
        {
            base.OnTick();
            if (this.status == STATUS_WAITING)
            {
                if (this.pendingOrder1.TradeType == TradeType.Buy)
                {
                    if (Symbol.Ask <= this.stopLossPrice)
                    {
                        CancelPendingOrderAsync(this.pendingOrder1);
                        CancelPendingOrderAsync(this.pendingOrder2);
                        this.status = STATUS_IDLE;
                    }
                }
                else
                {
                    if (Symbol.Bid >= this.stopLossPrice)
                    {
                        CancelPendingOrderAsync(this.pendingOrder1);
                        CancelPendingOrderAsync(this.pendingOrder2);
                        this.status = STATUS_IDLE;
                    }
                }
            }
        }

        protected override void OnBar()
        {
            base.OnBar();
            /*
            if (this.RunningMode == RunningMode.VisualBacktesting)
            {
                Chart.DrawIcon(Guid.NewGuid().ToString(), ChartIconType.Circle, Bars.OpenTimes.Last(1), this.ema21H1.Result.LastValue, Color.Red);
                Chart.DrawIcon(Guid.NewGuid().ToString(), ChartIconType.Circle, Bars.OpenTimes.Last(1), this.ema8H1.Result.LastValue, Color.Cyan);
            }
            */

            if (this.status == STATUS_WAITING)
            {
                if (this.pendingOrder1.TradeType == TradeType.Buy)
                {
                    if (Bars.ClosePrices.Last(1) < this.ema21M5.Result.Last(1))
                    {
                        CancelPendingOrderAsync(this.pendingOrder1);
                        CancelPendingOrderAsync(this.pendingOrder2);
                        this.status = STATUS_IDLE;
                    }
                }
                else
                {
                    if (Bars.ClosePrices.Last(1) > this.ema21M5.Result.Last(1))
                    {
                        CancelPendingOrderAsync(this.pendingOrder1);
                        CancelPendingOrderAsync(this.pendingOrder2);
                        this.status = STATUS_IDLE;
                    }
                }
            }

            if (this.status == STATUS_IDLE && this.isTradingHours(Bars.OpenTimes.LastValue))
            {
                if (this.buyConditions())
                {
                    Print("Buy conditions");
                    this.stopLossPrice = Bars.LowPrices.Last(1) - 3 * Symbol.PipSize;

                    double maxPrice = Symbol.Ask;
                    for (int i = 1; i <= 6; i++)
                    {
                        maxPrice = Math.Max(maxPrice, Bars.HighPrices.Last(i));
                    }
                    double entryPrice = maxPrice + 3 * Symbol.PipSize;
                    double takeProfitPrice1 = entryPrice + (entryPrice - stopLossPrice);
                    double takeProfitPrice2 = entryPrice + 2 * (entryPrice - stopLossPrice);

                    double stopLoss = entryPrice - stopLossPrice;
                    double takeProfit1 = takeProfitPrice1 - entryPrice;
                    double takeProfit2 = takeProfitPrice2 - entryPrice;

                    if (this.RunningMode == RunningMode.VisualBacktesting)
                    {
                        Chart.DrawIcon(Guid.NewGuid().ToString(), ChartIconType.UpArrow, Bars.OpenTimes.Last(1), Bars.LowPrices.Last(1) - 5 * Symbol.PipSize, Color.Yellow);
                    }
                    double volume = this.calculateVolume(stopLoss);
                    // Buy
                    TradeResult result = PlaceStopOrder(TradeType.Buy, Symbol.Name, volume, entryPrice, this.label, stopLoss / Symbol.PipSize, takeProfit1 / Symbol.PipSize);
                    if (!result.IsSuccessful)
                    {
                        Print("Error when placing a buy stop order");
                    }
                    else
                    {
                        this.pendingOrder1 = result.PendingOrder;
                        Print("Creada orden pendiente 1: " + this.pendingOrder1.Id);
                        this.status = STATUS_WAITING;
                    }

                    result = PlaceStopOrder(TradeType.Buy, Symbol.Name, volume, entryPrice, this.label, stopLoss / Symbol.PipSize, takeProfit2 / Symbol.PipSize);
                    if (!result.IsSuccessful)
                    {
                        Print("Error when placing a buy stop order");
                    }
                    else
                    {
                        this.pendingOrder2 = result.PendingOrder;
                        Print("Creada orden pendiente 2: " + this.pendingOrder2.Id);
                        this.status = STATUS_WAITING;
                    }
                }

                if (this.sellConditions())
                {
                    Print("Sell conditions");
                    this.stopLossPrice = Bars.HighPrices.Last(1) + 3 * Symbol.PipSize;

                    double minPrice = Symbol.Ask;
                    for (int i = 1; i <= 6; i++)
                    {
                        minPrice = Math.Min(minPrice, Bars.LowPrices.Last(i));
                    }
                    double entryPrice = minPrice - 3 * Symbol.PipSize;
                    double takeProfitPrice1 = entryPrice - (stopLossPrice - entryPrice);
                    double takeProfitPrice2 = entryPrice - 2 * (stopLossPrice - entryPrice);

                    double stopLoss = stopLossPrice - entryPrice;
                    double takeProfit1 = entryPrice - takeProfitPrice1;
                    double takeProfit2 = entryPrice - takeProfitPrice2;

                    if (this.RunningMode == RunningMode.VisualBacktesting)
                    {
                        Chart.DrawIcon(Guid.NewGuid().ToString(), ChartIconType.DownArrow, Bars.OpenTimes.Last(1), Bars.HighPrices.Last(1) + 5 * Symbol.PipSize, Color.Yellow);
                    }
                    double volume = this.calculateVolume(stopLoss);
                    // Buy
                    TradeResult result = PlaceStopOrder(TradeType.Sell, Symbol.Name, volume, entryPrice, this.label, stopLoss / Symbol.PipSize, takeProfit1 / Symbol.PipSize);
                    if (!result.IsSuccessful)
                    {
                        Print("Error when placing a sell stop order");
                    }
                    else
                    {
                        this.pendingOrder1 = result.PendingOrder;
                        Print("Creada orden pendiente 1: " + this.pendingOrder1.Id);
                        this.status = STATUS_WAITING;
                    }

                    result = PlaceStopOrder(TradeType.Sell, Symbol.Name, volume, entryPrice, this.label, stopLoss / Symbol.PipSize, takeProfit2 / Symbol.PipSize);
                    if (!result.IsSuccessful)
                    {
                        Print("Error when placing a sell stop order");
                    }
                    else
                    {
                        this.pendingOrder2 = result.PendingOrder;
                        Print("Creada orden pendiente 2: " + this.pendingOrder2.Id);
                        this.status = STATUS_WAITING;
                    }
                }
            }
        }

        private bool buyConditions()
        {
            double price = (Symbol.Bid + Symbol.Ask) / 2.0;
            bool buyH1 = price > this.ema8H1.Result.LastValue && this.ema8H1.Result.LastValue > this.ema21H1.Result.LastValue && this.ema8H1.Result.Last(1) > this.ema21H1.Result.Last(1);

            bool emasSorted = true;
            for (int i = 1; i <= 6; i++)
            {
                emasSorted = emasSorted && this.ema8M5.Result.Last(i) > this.ema13M5.Result.Last(i) && this.ema13M5.Result.Last(i) > this.ema21M5.Result.Last(i);
            }

            bool previousBars = true;
            for (int i = 2; i <= 6; i++)
            {
                previousBars = previousBars && Bars.LowPrices.Last(i) > this.ema8M5.Result.Last(i);
            }

            double low = Bars.LowPrices.Last(1);
            double high = Bars.HighPrices.Last(1);
            bool retrace = high > this.ema8M5.Result.Last(1) && this.ema8M5.Result.Last(1) > low && low > this.ema21M5.Result.Last(1);
            return buyH1 && previousBars && emasSorted && retrace;
        }

        private bool sellConditions()
        {
            double price = (Symbol.Bid + Symbol.Ask) / 2.0;
            bool sellH1 = price < this.ema8H1.Result.LastValue && this.ema8H1.Result.LastValue < this.ema21H1.Result.LastValue && this.ema8H1.Result.Last(1) < this.ema21H1.Result.Last(1);

            bool emasSorted = true;
            for (int i = 1; i <= 6; i++)
            {
                emasSorted = emasSorted && this.ema8M5.Result.Last(i) < this.ema13M5.Result.Last(i) && this.ema13M5.Result.Last(i) < this.ema21M5.Result.Last(i);
            }

            bool previousBars = true;
            for (int i = 2; i <= 6; i++)
            {
                previousBars = previousBars && Bars.HighPrices.Last(i) < this.ema8M5.Result.Last(i);
            }

            double low = Bars.LowPrices.Last(1);
            double high = Bars.HighPrices.Last(1);
            bool retrace = low < this.ema8M5.Result.Last(1) && this.ema8M5.Result.Last(1) < high && high < this.ema21M5.Result.Last(1);
            return sellH1 && previousBars && emasSorted && retrace;
        }

        private double calculateVolume(double stopLoss)
        {
            double vol = this.volume;
            switch (this.ParamMoneyMgmntType)
            {
                case MMType.FIXED_LOT:
                    vol = this.volume;
                    break;

                case MMType.RISK:
                    double conversionRate = 1.0;
                    if (Account.Currency.Equals("EUR"))
                    {
                        Symbol symbolEURYYY = Symbols.GetSymbol("EUR" + Symbol.Name.Substring(3, 3));
                        conversionRate = symbolEURYYY.Bid;
                    }

                    vol = this.normalizeVolume(Account.Equity * this.ParamRisk * conversionRate / (100.0 * stopLoss));
                    break;
            }
            return vol;
        }

        private double normalizeVolume(double vol)
        {
            double result = 1000;
            while (result + 1000 < vol)
            {
                result += 1000;
            }
            return result;
        }

        private void OnClosedPosition(PositionClosedEventArgs args)
        {
            if (args.Position.SymbolName.Equals(Symbol.Name) && this.label.Equals(args.Position.Label))
            {
                Print("Closed position " + args.Position.Id + " with net profit " + args.Position.NetProfit);

                if (args.Reason == PositionCloseReason.StopLoss || args.Reason == PositionCloseReason.Closed)
                {
                    this.status = STATUS_IDLE;
                }
                else if (args.Reason == PositionCloseReason.TakeProfit)
                {
                    Print("Take profit");
                    if (args.Position.Id == this.position1.Id)
                    {
                        Print("Take profit 1");
                        this.position2.ModifyStopLossPrice(this.position2.EntryPrice);
                    }
                    if (args.Position.Id == this.position2.Id)
                    {
                        Print("Take profit 2");
                        this.position1 = null;
                        this.position2 = null;
                        this.status = STATUS_IDLE;
                    }
                }
            }
        }

        private void OnPendingOrderFilled(PendingOrderFilledEventArgs args)
        {
            if (args.PendingOrder.SymbolName.Equals(Symbol.Name) && this.label.Equals(args.Position.Label))
            {
                this.status = STATUS_TRADING;

                if (this.pendingOrder1 != null && args.PendingOrder.Id == this.pendingOrder1.Id)
                {
                    this.pendingOrder1 = null;
                    this.position1 = args.Position;
                    Print("Creada posicion 1: " + this.position1.Id);
                }
                if (this.pendingOrder2 != null && args.PendingOrder.Id == this.pendingOrder2.Id)
                {
                    this.pendingOrder2 = null;
                    this.position2 = args.Position;
                    Print("Creada posicion 2: " + this.position2.Id);
                }
            }
        }

        private bool isTradingHours(DateTime dt)
        {
            if (!this.ParamRestrictedTradingHours)
            {
                return true;
            }
            BoxHours boxHours = this.getBoxHours(dt);
            int startMin = boxHours.startHour * 60 + boxHours.startMinute;
            int endMin = boxHours.endHour * 60 + boxHours.endMinute;
            int nowMin = dt.Hour * 60 + dt.Minute;
            bool result = false;
            if (startMin < endMin)
            {
                result = startMin <= nowMin && nowMin <= endMin;
            }
            else
            {
                result = endMin <= nowMin || nowMin <= startMin;
            }
            return result;
        }

        private BoxHours getBoxHours(DateTime dt)
        {
            BoxHours bh = new BoxHours();
            if (this.ParamAdjustSummerTime)
            {
                if (this.isSummer(dt))
                {
                    bh.startHour = this.ParamStartHour - 1;
                    if (bh.startHour < 0)
                    {
                        bh.startHour += 24;
                    }
                    bh.endHour = this.ParamEndHour - 1;
                    if (bh.endHour < 0)
                    {
                        bh.endHour += 24;
                    }
                }
                else
                {
                    bh.startHour = this.ParamStartHour;
                    bh.endHour = this.ParamEndHour;
                }
            }
            else
            {
                bh.startHour = this.ParamStartHour;
                bh.endHour = this.ParamEndHour;
            }
            bh.startMinute = this.ParamStartMinute;
            bh.endMinute = this.ParamEndMinute;
            return bh;
        }

        private bool isSummer(DateTime dt)
        {
            // Winter: November to March
            // Summer: April to October
            return dt.Month >= 4 && dt.Month <= 10;
        }
    }

    public class BoxHours
    {
        public int startHour { get; set; }
        public int startMinute { get; set; }
        public int endHour { get; set; }
        public int endMinute { get; set; }
    }

    public enum Status
    {
        IDLE,
        TRADING
    }

    public enum MMType
    {
        FIXED_LOT,
        RISK
    }
}
